%--------------------------------------------------------------------------
function  U = chambolle_fbs(F,tau,eps,lambda)
%--------------------------------------------------------------------------
% 1/2 \|f-u\|_2^2 + lambda TV(u) -> min
% via dual problem 
% FBS applied to for p = argmin_{\| |p| \|_infty \le lambda} 1/2 \| f - div p\|_2^2
% F: given (N x N) - matrix
% eps: stopping criterion
% lambda: regularization parameter
%tau < 0.25;
%--------------------------------------------------------------------------
t = cputime;

[N,N] = size(F);
NN = 2*N;

% Erstellung der Differenzenmatrix D
e = ones(N,1); 
D = spdiags([zeros(N,1) -1*e e], -1:1, N, N); 
D(N,N) = 0;

% Initialisierung
%tau = 0.2;
V = zeros(NN,N);
U = F;

k = 1;
stopping_criterion = inf;

for k=1:3000

  U_old = U;

  W = [D*U;U*D'];
  p1 = V(1:N,1:N) + tau* W(1:N,1:N);
  p2 = V(N+1:NN,1:N) + tau* W(N+1:NN,1:N);
  q = sqrt( p1.^2 + p2.^2)/lambda;
  V = [p1;p2]./max(ones(NN,N),[q;q]);

  U = F - D'*V(1:N,1:N) - V(N+1:NN,1:N)*D;

  stopping_criterion(k) = norm(U-U_old,'fro')/norm(U,'fro');
  if mod(k,10)==0
     disp(['Ite: ',num2str(k),',   Error C: ', num2str(stopping_criterion(k)), '!']);
  end
  
  if stopping_criterion(k)<eps
      break;
  end
  
%   stopping_criterion = max(max(abs(U - U_vgl)))
end
k
tt = cputime-t

showpic(U);
figure,plot(stopping_criterion),title('error of u');

%-----------------------------------------
function showpic(B)
%-----------------------------------------
figure();
imagesc(B);
colormap(gray);
colorbar;
axis square;
axis off;

